#include "DeflateEncoder.h"

#include "BitStream.h"
#include "DeflateBlock.h"
#include "BufferBitStream.h"

#include <iostream>

DeflateEncoder::DeflateEncoder(BitStream* inputStream, BitStream* outputStream)
    : AbstractEncoder(inputStream, outputStream)
{

}

DeflateEncoder::~DeflateEncoder()
{

}

bool DeflateEncoder::encode()
{
    uint16_t count = 0;
    BufferBitStream stream;
    std::unique_ptr<DeflateBlock> working_block = std::make_unique<DeflateBlock>(&stream, mOutputStream);

    AbstractChecksumCalculator* checksum_calc;
    if (mChecksumCalculators.size() > 0)
    {
       std::cout << "Setting checksum calculator " << std::endl;
       mOutputStream->setChecksumCalculator(mChecksumCalculators[0]);
    }

    while(true)
    {
        if (count == mMaxBlockSize)
        {
            std::cout << working_block->getMetaData();
            working_block->write(count);

            working_block = std::make_unique<DeflateBlock>(&stream, mOutputStream);
            stream.reset();
        }

        if (auto byte = mInputStream->readNextByte())
        {
            stream.writeByte(*byte);
        }
        else
        {
            stream.resetOffsets();
            working_block->setIsFinalBlock(true);

            std::cout << working_block->getMetaData();
            working_block->write(count);
            break;
        }
        count++;
    }

    mOutputStream->clearChecksumCalculator();
    return true;
}

bool DeflateEncoder::decode()
{
    auto working_block = std::make_unique<DeflateBlock>(mInputStream, mOutputStream);
    working_block->readHeader();

    DeflateBlock* raw_block = working_block.get();

    while(!raw_block->isFinalBlock())
    {
        break;
    }

    return false;
}

