#include "BitStream.h"

#include "ByteUtils.h"

#include <sstream>
#include <iostream>

BitStream::~BitStream()
{

}

unsigned char BitStream::getCurrentByte()
{
    if (mByteOffset < 0)
    {
        readNextByte();
    }
    return mCurrentByte;
}

void BitStream::write(uint32_t data)
{
    unsigned num_bytes = sizeof(uint32_t);
    for(unsigned idx=0; idx<num_bytes;idx++)
    {
        writeByte(ByteUtils::getByteN(data, idx));
    }
}

void BitStream::writeWord(uint16_t data)
{
    unsigned num_bytes = sizeof(uint16_t);
    for(unsigned idx=0; idx<num_bytes;idx++)
    {
        writeByte(ByteUtils::getByteN(data, idx));
    }
}

int BitStream::getCurrentByteOffset() const
{
    return mByteOffset;
}

unsigned BitStream::getCurrentBitOffset() const
{
    return mBitOffset;
}

std::string BitStream::logNextNBytes(unsigned n) const
{
    std::stringstream sstr;
    unsigned count{0};
    for(auto byte : peekNextNBytes(n))
    {
        sstr << count << " | " << ByteUtils::toString(byte) + '\n';
        count++;
    }
    return sstr.str();
}

bool BitStream::readNextNBits(unsigned n, unsigned char& buffer)
{
    if (mByteOffset < 0)
    {
        if (!readNextByte())
        {
            return false;
        }
    }

    int overshoot = n + mBitOffset - 8;
    if (overshoot > 0)
    {
        unsigned char last_byte = mCurrentByte;
        if (!readNextByte())
        {
            return false;
        }

        auto num_lower = 8 - mBitOffset;
        char lower_bits = ByteUtils::getHigherNBits(last_byte, num_lower);
        char higher_bits = ByteUtils::getLowerNBits(mCurrentByte, overshoot);

        buffer = (higher_bits << num_lower) | lower_bits;

        mBitOffset = overshoot;
        return true;
    }
    else
    {
        buffer = ByteUtils::getMBitsAtN(mCurrentByte, n, mBitOffset);
        mBitOffset += n;
        return true;
    }
}
