#pragma once

#include "AbstractChecksumCalculator.h"

#include <vector>
#include <string>
#include <optional>

class BitStream
{
public:
    virtual ~BitStream();

    unsigned char getCurrentByte();

    int getCurrentByteOffset() const;

    unsigned getCurrentBitOffset() const;

    virtual bool isFinished() const = 0;

    std::string logNextNBytes(unsigned n) const;

    virtual std::vector<unsigned char> peekNextNBytes(unsigned n) const = 0;

    virtual bool readNextNBits(unsigned n, unsigned char& buffer);

    virtual std::optional<unsigned char> readNextByte() = 0;

    virtual void writeByte(unsigned char data) = 0;

    void write(uint32_t data);

    void writeWord(uint16_t data);

    virtual void writeBytes(const std::vector<unsigned char> data) = 0;

    void resetOffsets()
    {
        mByteOffset = -1;
        mBitOffset = 0;
    }

    virtual void reset()
    {
        resetOffsets();
        mCurrentByte = 0;
    }

    void setChecksumCalculator(AbstractChecksumCalculator* calc)
    {
        mChecksumCalculator = calc;
    }

    void clearChecksumCalculator()
    {
        mChecksumCalculator = nullptr;
    }

    //unsigned getSize() = 0;

protected:
    int mByteOffset{-1};
    unsigned mBitOffset{0};

    unsigned char mCurrentByte{0};

    AbstractChecksumCalculator* mChecksumCalculator{nullptr};
};
