#include "FontsManager.h"

#include "FreeTypeFontEngine.h"
#include "BasicFontEngine.h"


FontsManager::FontsManager()
    : mFontEngine(std::make_unique<FreeTypeFontEngine>())
{
    mFontEngine->initialize();
}

std::unique_ptr<FontsManager> FontsManager::Create()
{
    return std::make_unique<FontsManager>();
}

IFontEngine* FontsManager::getFontEngine() const
{
    return mFontEngine.get();
}

FontGlyph* FontsManager::getGlyph(const std::string& fontFace, int size, char c)
{
    auto path = std::filesystem::path("truetype/msttcorefonts/arial.ttf");

    mFontEngine->loadFontFace(path, size);

    auto iter = mGlyphs.find(c);
    if(iter != mGlyphs.end())
    {
        return iter->second.get();
    }
    else
    {
        auto glyph = mFontEngine->loadGlyph(c);
        auto glyph_ptr = glyph.get();
        mGlyphs[c] = std::move(glyph);
        return glyph_ptr;
    }
}


