#include "PngReader.h"

#include "BinaryStream.h"
#include "BitStream.h"
#include "BufferBitStream.h"

#include "ZlibEncoder.h"

#include <iostream>
#include <sstream>

PngReader::~PngReader()
{

}

void PngReader::setPath(const Path& path)
{
    mPath = path;
}

bool PngReader::checkSignature()
{
    const int highBitCheck = 0x89;
    const auto firstPos = mFile->GetInHandle()->get();
    if (firstPos != highBitCheck)
    {
        return false;
    }

    std::string fileType;
    BinaryStream::getNextString(mFile->GetInHandle(), fileType, 3);
    if (fileType != "PNG")
    {
        return false;
    }

    std::vector<char> sequence{13, 10, 26, 10};
    for (auto c : sequence)
    {
        if (mFile->GetInHandle()->get() != c)
        {
            return false;
        }
    }

    mCurrentOffset += 8;
    return true;
}

bool PngReader::readChunk()
{
    unsigned length = *BinaryStream::getNextDWord(mFile->GetInHandle());

    std::string chunkType;
    BinaryStream::getNextString(mFile->GetInHandle(), chunkType, 4);
    mCurrentOffset += 8;

    std::cout << "Got chunk with type: " << chunkType << " and length: " << length << std::endl;
    bool lastChunk = false;
    if (chunkType == "IHDR")
    {
        readHeaderChunk();
    }
    else if(chunkType == "IEND")
    {
        lastChunk = true;
    }
    else if(chunkType == "IDAT")
    {
        readIDATChunk(length);
    }
    else
    {
        for(unsigned idx=0;idx<length;idx++)
        {
            mFile->GetInHandle()->get();
        }
    }

    unsigned crcCheck = *BinaryStream::getNextDWord(mFile->GetInHandle());

    mCurrentOffset += 4;
    return !lastChunk;
}

void PngReader::readIDATChunk(unsigned length)
{
    for(unsigned idx=0; idx<length; idx++)
    {
        mInputStream->writeByte(*mFile->readNextByte());
    }
}

void PngReader::readHeaderChunk()
{
    auto width = *BinaryStream::getNextDWord(mFile->GetInHandle());
    auto height = *BinaryStream::getNextDWord(mFile->GetInHandle());
    auto bitDepth = mFile->GetInHandle()->get();

    mHeader.setImageData(width, height, bitDepth);

    PngInfo info;
    info.mColorType = static_cast<PngInfo::ColorType>(mFile->GetInHandle()->get());
    info.mCompressionMethod = static_cast<PngInfo::CompressionMethod>(mFile->GetInHandle()->get());
    info.mFilterMethod = static_cast<PngInfo::FilterMethod>(mFile->GetInHandle()->get());
    info.mInterlaceMethod = static_cast<PngInfo::InterlaceMethod>(mFile->GetInHandle()->get());
    mHeader.setPngInfo(info);

    mCurrentOffset += 13;

    logHeader();
}

void PngReader::logHeader()
{
    std::cout << "IHDR\n" << mHeader.toString() << "*************\n";
}

std::unique_ptr<Image<unsigned char> > PngReader::read()
{
    auto image = std::make_unique<Image<unsigned char> >(5, 5);

    mFile = std::make_unique<File>(mPath);
    mFile->Open(true);

    if (!checkSignature())
    {
        std::cout << "Signature check failed" << std::endl;
        return image;
    }

    mInputStream = std::make_unique<BufferBitStream>();
    mOutputStream = std::make_unique<BufferBitStream>();
    mEncoder = std::make_unique<ZlibEncoder>(mInputStream.get(), mOutputStream.get());

    while(readChunk())
    {

    }

    std::cout << mEncoder->getData() << std::endl;
    mEncoder->decode();
    return std::move(image);
}
